@@ -1,7 +1,12 @@
 import logging
+from typing import Union, List, Dict, Optional, cast
 import time
 import socket
-from ssl import CERT_NONE, SSLError, CertificateError, create_default_context
+from ssl import (
+    CERT_NONE,
+    SSLContext,
+    create_default_context,
+)
 
 import imapclient
 import imapclient.exceptions
@@ -16,28 +21,42 @@ class MaxRetriesExceeded(RuntimeError):
     """Raised when the maximum number of retries in exceeded"""
 
 
-def _chunks(list_like_object, n):
+def _chunks(list_like_object, n: int):
     """Yield successive n-sized chunks from l."""
     for i in range(0, len(list_like_object), n):
-        yield list_like_object[i:i + n]
+        yield list_like_object[i : i + n]
 
 
 class IMAPClient(imapclient.IMAPClient):
     """A simplified IMAP client"""
 
-    def _normalise_folder(self, folder_name):
+    def _normalise_folder(
+        self, folder_name: Union[str, bytes, bytearray, memoryview]
+    ) -> str:
         """
         Returns an appropriate path based on the namespace (if any) and
         hierarchy separator
 
         Args:
-            folder_name (str): The path to correct
+            folder_name: The path to correct
 
         Returns:
-            str: A corrected path
+            A corrected path
         """
+        if isinstance(folder_name, memoryview):
+            folder_name = folder_name.tobytes()
+
+        if isinstance(folder_name, (bytes, bytearray)):
+            folder_name = bytes(folder_name).decode("utf-8", "replace")
+
+        folder_name = cast(str, folder_name)
+
         if folder_name in ["", "*", "INBOX"]:
-            return imapclient.IMAPClient._normalise_folder(self, folder_name)
+            result = imapclient.IMAPClient._normalise_folder(self, folder_name)
+            if isinstance(result, bytes):
+                return result.decode("utf-8", "replace")
+            return str(result)
+
         folder_name = folder_name.rstrip("/")
         folder_name = folder_name.replace(self._path_prefix, "")
         if not self._hierarchy_separator == "/":
@@ -45,19 +64,23 @@ class IMAPClient(imapclient.IMAPClient):
             folder_name = folder_name.replace("/", self._hierarchy_separator)
         folder_name = "{0}{1}".format(self._path_prefix, folder_name)
 
-        return imapclient.IMAPClient._normalise_folder(self, folder_name)
+        result = imapclient.IMAPClient._normalise_folder(self, folder_name)
+        if isinstance(result, bytes):
+            return result.decode("utf-8", "replace")
+        return str(result)
 
-    def _start_idle(self, idle_callback, idle_timeout=30):
+    def _start_idle(self, idle_callback, idle_timeout: int = 30):
         """
         Starts an IMAP IDLE session
 
         Args:
-            idle_callback (function: A callback function
-            idle_timeout (int): Number of seconds to wait for an IDLE response
+            idle_callback: A callback function
+            idle_timeout: Number of seconds to wait for an IDLE response
         """
         if self._idle_supported is False:
             raise imapclient.exceptions.IMAPClientError(
-                "IDLE is not supported by the server")
+                "IDLE is not supported by the server"
+            )
         idle_callback(self)
         idle_start_time = time.monotonic()
         self.idle()
@@ -68,7 +91,7 @@ class IMAPClient(imapclient.IMAPClient):
                     logger.info("IMAP: Refreshing IDLE session")
                     self.idle_done()
                     idle_start_time = time.monotonic()
-                    self.idle(self)
+                    self.idle()
                 responses = self.idle_check(timeout=idle_timeout)
                 if responses is not None:
                     if len(responses) == 0:
@@ -79,19 +102,18 @@ class IMAPClient(imapclient.IMAPClient):
                         self.idle()
                     else:
                         for r in responses:
-                            if r[0] != 0 and r[1] == b'RECENT':
+                            if r[0] != 0 and r[1] == b"RECENT":
                                 self.idle_done()
                                 idle_callback(self)
                                 idle_start_time = time.monotonic()
                                 self.idle()
                                 break
-            except (KeyError, socket.error, BrokenPipeError,
-                    ConnectionResetError):
+            except (KeyError, socket.error, BrokenPipeError, ConnectionResetError):
                 logger.debug("IMAP error: Connection reset")
                 self.reset_connection()
             except imapclient.exceptions.IMAPClientError as error:
                 error = error.__str__().lstrip("b'").rstrip("'").rstrip(".")
-                # Workaround for random Exchange/Office365 IMAP errors
+                # Workaround for random Exchange/Microsoft 365 IMAP errors
                 if "unexpected response" in error or "BAD" in error:
                     self.reset_connection()
             except KeyboardInterrupt:
@@ -101,25 +123,37 @@ class IMAPClient(imapclient.IMAPClient):
         except BrokenPipeError:
             pass
 
-    def __init__(self, host, username, password, port=None, ssl=True,
-                 ssl_context=None, verify=True, timeout=30, max_retries=4,
-                 initial_folder="INBOX", idle_callback=None, idle_timeout=30):
+    def __init__(
+        self,
+        host: str,
+        username: str,
+        password: str,
+        port: int = 933,
+        ssl: bool = True,
+        ssl_context: Optional[SSLContext] = None,
+        verify: bool = True,
+        timeout: int = 30,
+        max_retries: int = 4,
+        initial_folder: str = "INBOX",
+        idle_callback=None,
+        idle_timeout: int = 30,
+    ):
         """
         Connects to an IMAP server
 
         Args:
-            host (str): The server hostname or IP address
-            username (str): The username
-            password (str): The password
-            port (int): The port
-            ssl (bool): Use SSL or TLS
-            ssl_context (SSLContext): For more advanced TLS options
-            verify (bool): Verify the SSL/TLS certificate
-            timeout (float): Number of seconds to wait for an operation
-            max_retries (int): The maximum number of retries after a timeout
-            initial_folder (str): The initial folder to select
+            host: The server hostname or IP address
+            username: The username
+            password: The password
+            port: The port
+            ssl: Use SSL or TLS
+            ssl_context: For more advanced TLS options
+            verify: Verify the SSL/TLS certificate
+            timeout: Number of seconds to wait for an operation
+            max_retries: The maximum number of retries after a timeout
+            initial_folder: The initial folder to select
             idle_callback: The function to call when new messages are detected
-            idle_timeout (int): Number of seconds to wait for an IDLE
+            idle_timeout: Number of seconds to wait for an IDLE
                                   response
         """
 
@@ -128,14 +162,20 @@ class IMAPClient(imapclient.IMAPClient):
         if verify is False:
             ssl_context.check_hostname = False
             ssl_context.verify_mode = CERT_NONE
-        self._init_args = dict(host=host, username=username,
-                               password=password, port=port, ssl=ssl,
-                               ssl_context=ssl_context, verify=verify,
-                               timeout=timeout,
-                               max_retries=max_retries,
-                               initial_folder=initial_folder,
-                               idle_callback=idle_callback,
-                               idle_timeout=idle_timeout)
+        self._init_args = dict(
+            host=host,
+            username=username,
+            password=password,
+            port=port,
+            ssl=ssl,
+            ssl_context=ssl_context,
+            verify=verify,
+            timeout=timeout,
+            max_retries=max_retries,
+            initial_folder=initial_folder,
+            idle_callback=idle_callback,
+            idle_timeout=idle_timeout,
+        )
         self.max_retries = max_retries
         self.idle_callback = idle_callback
         self.idle_timeout = idle_timeout
@@ -143,13 +183,15 @@ class IMAPClient(imapclient.IMAPClient):
         self._hierarchy_separator = ""
         if not ssl:
             logger.info("Connecting to IMAP over plain text")
-        imapclient.IMAPClient.__init__(self,
-                                       host=host,
-                                       port=port,
-                                       ssl=ssl,
-                                       ssl_context=ssl_context,
-                                       use_uid=True,
-                                       timeout=timeout)
+        imapclient.IMAPClient.__init__(
+            self,
+            host=host,
+            port=port,
+            ssl=ssl,
+            ssl_context=ssl_context,
+            use_uid=True,
+            timeout=timeout,
+        )
         try:
             if not ssl and b"STARTTLS" in self.capabilities():
                 logger.info("IMAP server supports STARTTLS ... activating now")
@@ -162,9 +204,10 @@ class IMAPClient(imapclient.IMAPClient):
             self._hierarchy_separator = self.list_folders()[0][1]
             if not self._hierarchy_separator:
                 self._hierarchy_separator = ""
-            if type(self._hierarchy_separator) == bytes:
-                self._hierarchy_separator = self._hierarchy_separator.decode(
-                    "utf-8")
+            if type(self._hierarchy_separator) is bytes:
+                self._hierarchy_separator = bytes(self._hierarchy_separator).decode(
+                    "utf-8"
+                )
             if self._namespace:
                 self._namespace = self.namespace()
                 personal_namespace = self._namespace.personal
@@ -172,31 +215,19 @@ class IMAPClient(imapclient.IMAPClient):
                     self._hierarchy_separator = personal_namespace[0][1]
                     if not personal_namespace[0][0] == "":
                         self._path_prefix = personal_namespace[0][0]
-                        if type(self._path_prefix) == bytes:
-                            self._path_prefix = self._path_prefix.decode(
-                                "utf-8")
+                        if type(self._path_prefix) is bytes:
+                            self._path_prefix = self._path_prefix.decode("utf-8")
             else:
                 self._namespace = None
             self.select_folder(initial_folder)
-        except (ConnectionResetError, socket.error,
-                TimeoutError,
-                imapclient.exceptions.IMAPClientError) as error:
-            error = error.__str__().lstrip("b'").rstrip("'").rstrip(
-                ".")
+        except (
+            ConnectionResetError,
+            socket.error,
+            TimeoutError,
+            imapclient.exceptions.IMAPClientError,
+        ) as error:
+            error = error.__str__().lstrip("b'").rstrip("'").rstrip(".")
             raise imapclient.exceptions.IMAPClientError(error)
-        except ConnectionAbortedError:
-            raise imapclient.exceptions.IMAPClientError("Connection aborted")
-        except TimeoutError:
-            raise imapclient.exceptions.IMAPClientError("Connection timed out")
-        except SSLError as error:
-            raise imapclient.exceptions.IMAPClientError(
-                "SSL error: {0}".format(error.__str__()))
-        except CertificateError as error:
-            raise imapclient.exceptions.IMAPClientError(
-                "Certificate error: {0}".format(error.__str__()))
-        except BrokenPipeError:
-            raise imapclient.exceptions.IMAPClientError("Broken pipe")
-
         if idle_callback is not None:
             self._start_idle(idle_callback, idle_timeout=idle_timeout)
 
@@ -206,30 +237,32 @@ class IMAPClient(imapclient.IMAPClient):
         try:
             self.shutdown()
         except Exception as e:
-            logger.info(
-                "Failed to log out: {0}".format(e.__str__()))
-        self.__init__(self._init_args["host"],
-                      self._init_args["username"],
-                      self._init_args["password"],
-                      port=self._init_args["port"],
-                      ssl=self._init_args["ssl"],
-                      ssl_context=self._init_args["ssl_context"],
-                      verify=self._init_args["verify"],
-                      timeout=self._init_args["timeout"],
-                      max_retries=self._init_args["max_retries"],
-                      initial_folder=self._init_args["initial_folder"],
-                      idle_callback=self._init_args["idle_callback"],
-                      idle_timeout=self._init_args["idle_timeout"],
-                      )
+            logger.info("Failed to log out: {0}".format(e.__str__()))
+        self.__init__(
+            self._init_args["host"],  # pyright: ignore[reportArgumentType]
+            self._init_args["username"],  # pyright: ignore[reportArgumentType]
+            self._init_args["password"],  # pyright: ignore[reportArgumentType]
+            port=self._init_args["port"],  # pyright: ignore[reportArgumentType]
+            ssl=self._init_args["ssl"],  # pyright: ignore[reportArgumentType]
+            ssl_context=self._init_args["ssl_context"],
+            verify=self._init_args["verify"],  # pyright: ignore[reportArgumentType]
+            timeout=self._init_args["timeout"],  # pyright: ignore[reportArgumentType]
+            max_retries=self._init_args["max_retries"],  # pyright: ignore[reportArgumentType]
+            initial_folder=self._init_args["initial_folder"],  # pyright: ignore[reportArgumentType]
+            idle_callback=self._init_args["idle_callback"],
+            idle_timeout=self._init_args["idle_timeout"],  # pyright: ignore[reportArgumentType]
+        )
 
-    def fetch_message(self, msg_uid, parse=False, _attempt=1):
+    def fetch_message(
+        self, msg_uid: int, parse: bool = False, _attempt: int = 1
+    ) -> Union[str, Dict]:
         """
         Fetch a message by UID, and optionally parse it
 
         Args:
-            msg_uid (int): The message UID
-            parse (bool): Return parsed results from mailparser
-            _attempt (int): The attempt number
+            msg_uid: The message UID
+            parse: Return parsed results from mailparser
+            _attempt: The attempt number
 
         Returns:
             str: The raw mail message, including headers
@@ -241,116 +274,133 @@ class IMAPClient(imapclient.IMAPClient):
             _attempt = _attempt + 1
             if _attempt > self.max_retries:
                 raise MaxRetriesExceeded("Maximum retries exceeded")
-            logger.info("Attempt {0} of {1} timed out. Retrying...".format(
-                _attempt,
-                self.max_retries))
+            logger.info(
+                "Attempt {0} of {1} timed out. Retrying...".format(
+                    _attempt, self.max_retries
+                )
+            )
             self.reset_connection()
             return self.fetch_message(msg_uid, parse=parse, _attempt=_attempt)
-        msg_keys = [b'RFC822', b'BODY[NULL]', b'BODY[]']
-        msg_key = ''
+        msg_keys = [b"RFC822", b"BODY[NULL]", b"BODY[]"]
+        msg_key = ""
         for key in msg_keys:
             if key in raw_msg.keys():
                 msg_key = key
                 break
-        message = raw_msg[msg_key].decode("utf-8", "replace")
+        message = raw_msg[msg_key].decode("utf-8", "replace")  # pyright: ignore[reportOptionalMemberAccess, reportAttributeAccessIssue, reportArgumentType]
         if parse:
             message = mailsuite.utils.parse_email(message)
         return message
 
-    def delete_messages(self, msg_uids, silent=True, _attempt=1):
+    def delete_messages(
+        self,
+        messages: Union[List[int], List[str], str, int],
+        silent: bool = True,
+        _attempt: int = 1,
+    ):
         """
         Deletes the given messages by Message UIDs
 
         Args:
-            msg_uids (list): A list of UIDs of messages to delete
-            silent (bool): Do it silently
-            _attempt (int): The attempt number
+            messages: A list of UIDs of messages to delete
+            silent: Do it silently
+            _attempt: The attempt number
         """
-        logger.info("Deleting message UID(s) {0}".format(",".join(
-            str(uid) for uid in msg_uids)))
-        if type(msg_uids) == str or type(msg_uids) == int:
-            msg_uids = [int(msg_uids)]
+        logger.info(
+            "Deleting message UID(s) {0}".format(",".join(str(uid) for uid in messages))  # pyright: ignore[reportGeneralTypeIssues]
+        )
+        if type(messages) is str or type(messages) is int:
+            messages = [int(messages)]
         try:
-            imapclient.IMAPClient.delete_messages(self, msg_uids,
-                                                  silent=silent)
-            imapclient.IMAPClient.expunge(self, msg_uids)
+            imapclient.IMAPClient.delete_messages(self, messages, silent=silent)
+            imapclient.IMAPClient.expunge(self, messages)
         except (socket.timeout, imaplib.IMAP4.abort):
             _attempt = _attempt + 1
             if _attempt > self.max_retries:
                 raise MaxRetriesExceeded("Maximum retries exceeded")
-            logger.info("Attempt {0} of {1} timed out. Retrying...".format(
-                _attempt,
-                self.max_retries))
+            logger.info(
+                "Attempt {0} of {1} timed out. Retrying...".format(
+                    _attempt, self.max_retries
+                )
+            )
             self.reset_connection()
-            self.delete_messages(msg_uids, silent=silent, _attempt=_attempt)
+            self.delete_messages(messages, silent=silent, _attempt=_attempt)
 
-    def create_folder(self, folder_path, _attempt=1):
+    def create_folder(self, folder: str, _attempt: int = 1):
         """
         Creates an IMAP folder at the given path
 
         Args:
-            folder_path (str): The path of the folder to create
-            _attempt (int): The attempt number
+            folder: The path of the folder to create
+            _attempt: The attempt number
         """
-        if not self.folder_exists(folder_path):
-            logger.info("Creating folder: {0}".format(folder_path))
+        if not self.folder_exists(folder):
+            logger.info("Creating folder: {0}".format(folder))
             try:
-                imapclient.IMAPClient.create_folder(self, folder_path)
+                imapclient.IMAPClient.create_folder(self, folder)
             except (socket.timeout, imaplib.IMAP4.abort):
                 _attempt = _attempt + 1
                 if _attempt > self.max_retries:
                     raise MaxRetriesExceeded("Maximum retries exceeded")
-                logger.info("Attempt {0} of {1} timed out. Retrying...".format(
-                    _attempt,
-                    self.max_retries))
+                logger.info(
+                    "Attempt {0} of {1} timed out. Retrying...".format(
+                        _attempt, self.max_retries
+                    )
+                )
                 self.reset_connection()
-                self.create_folder(folder_path, _attempt=_attempt)
+                self.create_folder(folder, _attempt=_attempt)
 
-    def _move_messages(self, msg_uids, folder_path):
+    def _move_messages(self, msg_uids: Union[int, List[int]], folder_path: str):
         """
         Move the emails with the given UIDs to the given folder
 
         Args:
             msg_uids: A UID or list of UIDs of messages to move
-            folder_path (str): The path of the destination folder
+            folder_path: The path of the destination folder
         """
         folder_path = folder_path.replace("\\", "/").rstrip("/")
-        if type(msg_uids) == str or type(msg_uids) == int:
+        if type(msg_uids) is str or type(msg_uids) is int:
             msg_uids = [int(msg_uids)]
         for chunk in _chunks(msg_uids, 100):
             if self._move_supported:
-                logger.info("Moving message UID(s) {0} to {1}".format(
-                    ",".join(str(uid) for uid in chunk), folder_path
-                ))
+                logger.info(
+                    "Moving message UID(s) {0} to {1}".format(
+                        ",".join(str(uid) for uid in chunk), folder_path
+                    )
+                )
                 try:
                     self.move(chunk, folder_path)
                 except imapclient.exceptions.IMAPClientError as e:
-                    e = e.__str__().lstrip("b'").rstrip(
-                        "'").rstrip(".")
+                    e = e.__str__().lstrip("b'").rstrip("'").rstrip(".")
                     message = "Error moving message UIDs"
-                    e = "{0} {1}: " "{2}".format(message, msg_uids, e)
+                    e = "{0} {1}: {2}".format(message, msg_uids, e)
                     logger.info("IMAP error: {0}".format(e))
                     logger.info(
                         "Copying message UID(s) {0} to {1} by copy".format(
                             ",".join(str(uid) for uid in chunk), folder_path
-                        ))
+                        )
+                    )
                     self.copy(msg_uids, folder_path)
                     self.delete_messages(msg_uids)
             else:
-                logger.info("Moving message UID(s) {0} to {1} by copy".format(
-                    ",".join(str(uid) for uid in chunk), folder_path
-                ))
+                logger.info(
+                    "Moving message UID(s) {0} to {1} by copy".format(
+                        ",".join(str(uid) for uid in chunk), folder_path
+                    )
+                )
                 self.copy(msg_uids, folder_path)
                 self.delete_messages(msg_uids)
 
-    def move_messages(self, msg_uids, folder_path, _attempt=1):
+    def move_messages(
+        self, msg_uids: Union[int, List[int]], folder_path: str, _attempt: int = 1
+    ):
         """
         Move the emails with the given UIDs to the given folder
 
         Args:
             msg_uids: A UID or list of UIDs of messages to move
-            folder_path (str): The path of the destination folder
-            _attempt (int): The attempt number
+            folder_path: The path of the destination folder
+            _attempt: The attempt number
         """
         try:
             self._move_messages(msg_uids, folder_path)
@@ -358,8 +408,10 @@ class IMAPClient(imapclient.IMAPClient):
             _attempt = _attempt + 1
             if _attempt > self.max_retries:
                 raise MaxRetriesExceeded("Maximum retries exceeded")
-            logger.info("Attempt {0} of {1} timed out. Retrying...".format(
-                _attempt,
-                self.max_retries))
+            logger.info(
+                "Attempt {0} of {1} timed out. Retrying...".format(
+                    _attempt, self.max_retries
+                )
+            )
             self.reset_connection()
             self._move_messages(msg_uids, folder_path)
