@@ -2,7 +2,7 @@ import logging
 import socket
 import smtplib
 from ssl import SSLError, CertificateError, create_default_context, CERT_NONE
-from typing import List, Dict, Tuple
+from typing import Tuple, Optional
 
 from mailsuite.utils import create_email
 
@@ -13,14 +13,24 @@ class SMTPError(RuntimeError):
     """Raised when a SMTP error occurs"""
 
 
-def send_email(host: str, message_from: str, message_to: List[str] = None,
-               message_cc: List = None, message_bcc: List = None,
-               port: int = 0, require_encryption: bool = False,
-               verify: bool = True, username: str = None, password: str = None,
-               envelope_from: str = None, subject: str = None,
-               message_headers: Dict = None,
-               attachments: Tuple[str, bytes] = None,
-               plain_message: str = None, html_message: str = None):
+def send_email(
+    host: str,
+    message_from: str,
+    message_to: Optional[list[str]] = None,
+    message_cc: Optional[list] = None,
+    message_bcc: Optional[list] = None,
+    port: int = 0,
+    require_encryption: bool = False,
+    verify: bool = True,
+    username: Optional[str] = None,
+    password: Optional[str] = None,
+    envelope_from: Optional[str] = None,
+    subject: Optional[str] = None,
+    message_headers: Optional[dict] = None,
+    attachments: Optional[list[Tuple[str, bytes]]] = None,
+    plain_message: Optional[str] = None,
+    html_message: Optional[str] = None,
+):
     """
     Send an email using a SMTP relay
 
@@ -38,15 +48,21 @@ def send_email(host: str, message_from: str, message_to: List[str] = None,
         envelope_from: Overrides the SMTP envelope "mail from" header
         subject: The message subject
         message_headers: Custom message headers
-        attachments: A list of tuples, containing filenames as bytes
+        attachments: A list of tuples, containing filenames and bytes
         plain_message: The plain text message body
         html_message: The HTML message body
     """
-    msg = create_email(message_from=message_from, message_to=message_to,
-                       message_cc=message_cc, subject=subject,
-                       message_headers=message_headers,
-                       attachments=attachments,
-                       plain_message=plain_message, html_message=html_message)
+
+    msg = create_email(
+        message_from=message_from,
+        message_to=message_to,
+        message_cc=message_cc,
+        subject=subject,
+        message_headers=message_headers,
+        attachments=attachments,
+        plain_message=plain_message,
+        html_message=html_message,
+    )
 
     try:
         ssl_context = create_default_context()
@@ -65,12 +81,15 @@ def send_email(host: str, message_from: str, message_to: List[str] = None,
                 server.starttls(context=ssl_context)
                 server.ehlo()
             else:
-                logger.warning("SMTP server does not support STARTTLS. "
-                               "Proceeding in plain text!")
+                logger.warning(
+                    "SMTP server does not support STARTTLS. Proceeding in plain text!"
+                )
         if username and password:
             server.login(username, password)
         if envelope_from is None:
             envelope_from = message_from
+        if message_to is None and message_to is None:
+            raise ValueError("message_to and envelope_to cannot both be None")
         envelope_to = message_to.copy()
         if message_cc is not None:
             message_to += message_cc
@@ -91,7 +110,7 @@ def send_email(host: str, message_from: str, message_to: List[str] = None,
         raise SMTPError("Connection aborted")
     except TimeoutError:
         raise SMTPError("Connection timed out")
-    except SSLError as error:
-        raise SMTPError("SSL error: {0}".format(error.__str__()))
     except CertificateError as error:
         raise SMTPError("Certificate error: {0}".format(error.__str__()))
+    except SSLError as error:
+        raise SMTPError("SSL error: {0}".format(error.__str__()))
